@@ -185,6 +185,37 @@ create_txt_record (SalutAvahiSelf *self,
    return ret;
 }
 
+static gboolean
+salut_avahi_self_set_caps (SalutSelf *_self,
+                           GError **error)
+{
+  SalutAvahiSelf *self = SALUT_AVAHI_SELF (_self);
+  SalutAvahiSelfPrivate *priv = SALUT_AVAHI_SELF_GET_PRIVATE (self);
+
+  if (priv->presence == NULL)
+    /* Service is not announced yet */
+    return TRUE;
+
+  ga_entry_group_service_freeze (priv->presence);
+
+  if (_self->node == NULL)
+      ga_entry_group_service_remove_key (priv->presence, "node", NULL);
+  else
+      ga_entry_group_service_set (priv->presence, "node", _self->node, NULL);
+
+  if (_self->hash == NULL)
+      ga_entry_group_service_remove_key (priv->presence, "hash", NULL);
+  else
+      ga_entry_group_service_set (priv->presence, "hash", _self->hash, NULL);
+
+  if (_self->ver == NULL)
+      ga_entry_group_service_remove_key (priv->presence, "ver", NULL);
+  else
+      ga_entry_group_service_set (priv->presence, "ver", _self->ver, NULL);
+
+  return ga_entry_group_service_thaw (priv->presence, error);
+}
+
 static gboolean
 salut_avahi_self_announce (SalutSelf *_self,
                            gint port,
@@ -217,6 +248,9 @@ salut_avahi_self_announce (SalutSelf *_self,
   if (priv->presence == NULL)
     goto error;
 
+  if (!salut_avahi_self_set_caps (_self, NULL))
+    goto error;
+
   if (!ga_entry_group_commit (priv->presence_group, error))
     goto error;
 
@@ -229,19 +263,19 @@ error:
 }
 
 static gboolean
-salut_avahi_self_set_presence (SalutSelf *_self,
+salut_avahi_self_set_presence (SalutSelf *self,
                                GError **error)
 {
-  SalutAvahiSelf *self = SALUT_AVAHI_SELF (_self);
-  SalutAvahiSelfPrivate *priv = SALUT_AVAHI_SELF_GET_PRIVATE (self);
+  SalutAvahiSelf *avahi_self = SALUT_AVAHI_SELF (self);
+  SalutAvahiSelfPrivate *priv = SALUT_AVAHI_SELF_GET_PRIVATE (avahi_self);
 
   ga_entry_group_service_freeze (priv->presence);
   ga_entry_group_service_set (priv->presence, "status",
-      salut_presence_status_txt_names[_self->status], NULL);
+      salut_presence_status_txt_names[self->status], NULL);
 
-  if (_self->status_message)
+  if (self->status_message)
     ga_entry_group_service_set (priv->presence, "msg",
-        _self->status_message, NULL);
+        self->status_message, NULL);
   else
     ga_entry_group_service_remove_key (priv->presence, "msg", NULL);
 
@@ -440,6 +474,7 @@ salut_avahi_self_class_init (
 
   self_class->announce = salut_avahi_self_announce;
   self_class->set_presence = salut_avahi_self_set_presence;
+  self_class->set_caps = salut_avahi_self_set_caps;
   self_class->set_alias = salut_avahi_self_set_alias;
   self_class->remove_avatar = salut_avahi_self_remove_avatar;
   self_class->set_avatar = salut_avahi_self_set_avatar;
