@@ -1,6 +1,6 @@
 /*
  * salut-im-channel.c - Source for SalutImChannel
- * Copyright (C) 2005-2007 Collabora Ltd.
+ * Copyright (C) 2005-2008 Collabora Ltd.
  *   @author: Sjoerd Simons <sjoerd@luon.net>
  *
  * This library is free software; you can redistribute it and/or
@@ -18,62 +18,63 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <dbus/dbus-glib.h>
+#include "salut-im-channel.h"
+
+#include <errno.h>
+#include <netdb.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <sys/socket.h>
-#include <netdb.h>
-#include <errno.h>
 #include <string.h>
+#include <sys/socket.h>
+
+#include <dbus/dbus-glib.h>
+#include <telepathy-glib/channel-iface.h>
+#include <telepathy-glib/dbus.h>
+#include <telepathy-glib/exportable-channel.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/svc-generic.h>
+#include <telepathy-glib/text-mixin.h>
+
+#include <gibber/gibber-linklocal-transport.h>
+#include <gibber/gibber-namespaces.h>
+#include <gibber/gibber-xmpp-connection.h>
+#include <gibber/gibber-xmpp-stanza.h>
 
 #define DEBUG_FLAG DEBUG_IM
 #include "debug.h"
-
-#include "salut-im-channel.h"
-#include "signals-marshal.h"
-
 #include "salut-connection.h"
 #include "salut-contact.h"
-#include "text-helper.h"
 #include "salut-xmpp-connection-manager.h"
+#include "salut-signals-marshal.h"
+#include "text-helper.h"
 
-#include <gibber/gibber-linklocal-transport.h>
-#include <gibber/gibber-xmpp-connection.h>
-#include <gibber/gibber-xmpp-stanza.h>
-#include <gibber/gibber-namespaces.h>
+static void channel_iface_init (gpointer g_iface, gpointer iface_data);
+static void text_iface_init (gpointer g_iface, gpointer iface_data);
 
-#include <telepathy-glib/text-mixin.h>
-#include <telepathy-glib/channel-iface.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/dbus.h>
+static void xmpp_connection_manager_new_connection_cb (
+    SalutXmppConnectionManager *mgr, GibberXmppConnection *conn,
+    SalutContact *contact, gpointer user_data);
 
-static void
-channel_iface_init (gpointer g_iface, gpointer iface_data);
-static void
-text_iface_init (gpointer g_iface, gpointer iface_data);
-
-static void
-xmpp_connection_manager_new_connection_cb (SalutXmppConnectionManager *mgr,
-                                           GibberXmppConnection *conn,
-                                           SalutContact *contact,
-                                           gpointer user_data);
-
-static void
-_setup_connection (SalutImChannel *self);
+static void _setup_connection (SalutImChannel *self);
 
 G_DEFINE_TYPE_WITH_CODE (SalutImChannel, salut_im_channel, G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_DBUS_PROPERTIES,
+      tp_dbus_properties_mixin_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL);
+    G_IMPLEMENT_INTERFACE (TP_TYPE_EXPORTABLE_CHANNEL, NULL);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_TEXT, text_iface_init);
 );
 
-static gboolean
-message_stanza_filter (SalutXmppConnectionManager *mgr,
+static const gchar *salut_im_channel_interfaces[] = {
+    NULL
+};
+
+static gboolean message_stanza_filter (SalutXmppConnectionManager *mgr,
     GibberXmppConnection *conn, GibberXmppStanza *stanza,
     SalutContact *contact, gpointer user_data);
 
-static void
-message_stanza_callback (SalutXmppConnectionManager *mgr,
+static void message_stanza_callback (SalutXmppConnectionManager *mgr,
     GibberXmppConnection *conn, GibberXmppStanza *stanza,
     SalutContact *contact, gpointer user_data);
 
@@ -96,6 +97,13 @@ enum
   PROP_CONTACT,
   PROP_CONNECTION,
   PROP_XMPP_CONNECTION_MANAGER,
+  PROP_INTERFACES,
+  PROP_TARGET_ID,
+  PROP_INITIATOR_HANDLE,
+  PROP_INITIATOR_ID,
+  PROP_REQUESTED,
+  PROP_CHANNEL_PROPERTIES,
+  PROP_CHANNEL_DESTROYED,
   LAST_PROPERTY
 };
 
@@ -107,6 +115,7 @@ struct _SalutImChannelPrivate
   gboolean dispose_has_run;
   gchar *object_path;
   TpHandle handle;
+  TpHandle initiator;
   SalutContact *contact;
   SalutConnection *connection;
   GibberXmppConnection *xmpp_connection;
@@ -114,6 +123,7 @@ struct _SalutImChannelPrivate
   /* Outcoming and incoming message queues */
   GQueue *out_queue;
   ChannelState state;
+  gboolean closed;
 };
 
 #define SALUT_IM_CHANNEL_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE \
@@ -134,12 +144,15 @@ salut_im_channel_do_close (SalutImChannel *self)
   SalutImChannelPrivate *priv = SALUT_IM_CHANNEL_GET_PRIVATE (self);
   ChannelState oldstate = priv->state;
 
+  if (priv->closed)
+    return;
+  priv->closed = TRUE;
+
   priv->state = CHANNEL_NOT_CONNECTED;
 
   switch (oldstate)
     {
       case CHANNEL_NOT_CONNECTED:
-        /* FIXME return an error ? */
         break;
       case CHANNEL_CONNECTING:
         break;
@@ -207,6 +220,7 @@ salut_im_channel_get_property (GObject *object,
 {
   SalutImChannel *chan = SALUT_IM_CHANNEL (object);
   SalutImChannelPrivate *priv = SALUT_IM_CHANNEL_GET_PRIVATE (chan);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->connection;
 
   switch (property_id)
     {
@@ -231,6 +245,52 @@ salut_im_channel_get_property (GObject *object,
       case PROP_XMPP_CONNECTION_MANAGER:
         g_value_set_object (value, priv->xmpp_connection_manager);
         break;
+      case PROP_INTERFACES:
+        g_value_set_static_boxed (value, salut_im_channel_interfaces);
+        break;
+      case PROP_TARGET_ID:
+        {
+           TpHandleRepoIface *repo = tp_base_connection_get_handles (base_conn,
+             TP_HANDLE_TYPE_CONTACT);
+
+           g_value_set_string (value, tp_handle_inspect (repo, priv->handle));
+        }
+        break;
+      case PROP_INITIATOR_HANDLE:
+        g_value_set_uint (value, priv->initiator);
+        break;
+      case PROP_INITIATOR_ID:
+          {
+            TpHandleRepoIface *repo = tp_base_connection_get_handles (
+                base_conn, TP_HANDLE_TYPE_CONTACT);
+
+            g_assert (priv->initiator != 0);
+            g_value_set_string (value,
+                tp_handle_inspect (repo, priv->initiator));
+          }
+        break;
+      case PROP_REQUESTED:
+        g_value_set_boolean (value, (priv->initiator == base_conn->self_handle));
+        break;
+      case PROP_CHANNEL_DESTROYED:
+        /* TODO: this should be FALSE if there are still pending messages, so
+         *       the channel manager can respawn the channel.
+         */
+        g_value_set_boolean (value, TRUE);
+        break;
+      case PROP_CHANNEL_PROPERTIES:
+        g_value_take_boxed (value,
+            tp_dbus_properties_mixin_make_properties_hash (object,
+                TP_IFACE_CHANNEL, "TargetHandle",
+                TP_IFACE_CHANNEL, "TargetHandleType",
+                TP_IFACE_CHANNEL, "ChannelType",
+                TP_IFACE_CHANNEL, "TargetID",
+                TP_IFACE_CHANNEL, "InitiatorHandle",
+                TP_IFACE_CHANNEL, "InitiatorID",
+                TP_IFACE_CHANNEL, "Requested",
+                TP_IFACE_CHANNEL, "Interfaces",
+                NULL));
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
         break;
@@ -256,6 +316,10 @@ salut_im_channel_set_property (GObject *object,
       case PROP_HANDLE:
         priv->handle = g_value_get_uint (value);
         break;
+      case PROP_INITIATOR_HANDLE:
+        priv->initiator = g_value_get_uint (value);
+        g_assert (priv->initiator != 0);
+        break;
       case PROP_CONTACT:
         priv->contact = g_value_get_object (value);
         g_object_ref (priv->contact);
@@ -300,7 +364,7 @@ salut_im_channel_constructor (GType type,
 
   priv = SALUT_IM_CHANNEL_GET_PRIVATE (SALUT_IM_CHANNEL (obj));
 
-  /* Ref our handle */
+  /* Ref our handle and initiator handle */
   base_conn = TP_BASE_CONNECTION (priv->connection);
 
   contact_repo = tp_base_connection_get_handles (base_conn,
@@ -308,6 +372,9 @@ salut_im_channel_constructor (GType type,
 
   tp_handle_ref (contact_repo, priv->handle);
 
+  g_assert (priv->initiator != 0);
+  tp_handle_ref (contact_repo, priv->initiator);
+
   /* Initialize text mixin */
   tp_text_mixin_init (obj, G_STRUCT_OFFSET (SalutImChannel, text),
       contact_repo);
@@ -335,6 +402,25 @@ salut_im_channel_class_init (SalutImChannelClass *salut_im_channel_class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (salut_im_channel_class);
   GParamSpec *param_spec;
+  static TpDBusPropertiesMixinPropImpl channel_props[] = {
+      { "TargetHandleType", "handle-type", NULL },
+      { "TargetHandle", "handle", NULL },
+      { "TargetID", "target-id", NULL },
+      { "ChannelType", "channel-type", NULL },
+      { "Interfaces", "interfaces", NULL },
+      { "Requested", "requested", NULL },
+      { "InitiatorHandle", "initiator-handle", NULL },
+      { "InitiatorID", "initiator-id", NULL },
+      { NULL }
+  };
+  static TpDBusPropertiesMixinIfaceImpl prop_interfaces[] = {
+      { TP_IFACE_CHANNEL,
+        tp_dbus_properties_mixin_getter_gobject_properties,
+        NULL,
+        channel_props,
+      },
+      { NULL }
+  };
 
   g_type_class_add_private (salut_im_channel_class,
       sizeof (SalutImChannelPrivate));
@@ -353,16 +439,43 @@ salut_im_channel_class_init (SalutImChannelClass *salut_im_channel_class)
   g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
       "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
+  g_object_class_override_property (object_class, PROP_CHANNEL_DESTROYED,
+      "channel-destroyed");
+  g_object_class_override_property (object_class, PROP_CHANNEL_PROPERTIES,
+      "channel-properties");
+
+  param_spec = g_param_spec_string ("target-id", "Target JID",
+      "The string obtained by inspecting this channel's handle",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_TARGET_ID, param_spec);
+
+  param_spec = g_param_spec_boolean ("requested", "Requested?",
+      "True if this channel was requested by the local user",
+      FALSE,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_REQUESTED, param_spec);
+
+  param_spec = g_param_spec_uint ("initiator-handle", "Initiator's handle",
+      "The contact who initiated the channel",
+      0, G_MAXUINT32, 0,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INITIATOR_HANDLE,
+      param_spec);
+
+  param_spec = g_param_spec_string ("initiator-id", "Initiator JID",
+      "The string obtained by inspecting the initiator-handle",
+      NULL,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INITIATOR_ID,
+      param_spec);
 
   param_spec = g_param_spec_object (
       "contact",
       "SalutContact object",
       "Salut Contact to which this channel is dedicated",
       SALUT_TYPE_CONTACT,
-      G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONTACT, param_spec);
 
   param_spec = g_param_spec_object (
@@ -370,10 +483,7 @@ salut_im_channel_class_init (SalutImChannelClass *salut_im_channel_class)
       "SalutConnection object",
       "Salut Connection that owns the connection for this IM channel",
       SALUT_TYPE_CONNECTION,
-      G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
   param_spec = g_param_spec_object (
@@ -381,15 +491,22 @@ salut_im_channel_class_init (SalutImChannelClass *salut_im_channel_class)
       "SalutXmppConnectionManager object",
       "Salut XMPP Connection manager used for this IM channel",
       SALUT_TYPE_XMPP_CONNECTION_MANAGER,
-      G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS);
   g_object_class_install_property (object_class, PROP_XMPP_CONNECTION_MANAGER,
       param_spec);
 
+  param_spec = g_param_spec_boxed ("interfaces", "Extra D-Bus interfaces",
+      "Additional Channel.Interface.* interfaces",
+      G_TYPE_STRV,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_INTERFACES, param_spec);
+
   tp_text_mixin_class_init (object_class,
       G_STRUCT_OFFSET (SalutImChannelClass, text_class));
+
+  salut_im_channel_class->dbus_props_class.interfaces = prop_interfaces;
+  tp_dbus_properties_mixin_class_init (object_class,
+      G_STRUCT_OFFSET (SalutImChannelClass, dbus_props_class));
 }
 
 void
@@ -411,8 +528,10 @@ salut_im_channel_dispose (GObject *object)
 
   tp_handle_unref (handle_repo, priv->handle);
 
-  if (priv->state != CHANNEL_NOT_CONNECTED)
-    salut_im_channel_do_close (self);
+  if (priv->initiator != 0)
+    tp_handle_unref (handle_repo, priv->initiator);
+
+  salut_im_channel_do_close (self);
 
   if (priv->xmpp_connection)
     {
@@ -791,7 +910,6 @@ _send_message (SalutImChannel *self,
       case CHANNEL_NOT_CONNECTED:
       case CHANNEL_CONNECTING:
       case CHANNEL_CLOSING:
-        g_object_ref (stanza);
         msg = salut_im_channel_message_new (type, text, stanza);
         _send_channel_message (self, msg);
         break;
@@ -921,9 +1039,8 @@ static void
 salut_im_channel_get_interfaces (TpSvcChannel *iface,
                                  DBusGMethodInvocation *context)
 {
-  const char *interfaces[] = { NULL };
-
-  tp_svc_channel_return_from_get_interfaces (context, interfaces);
+  tp_svc_channel_return_from_get_interfaces (context,
+      salut_im_channel_interfaces);
 }
 
 static void
