@@ -38,6 +38,7 @@
 #include "salut-xmpp-connection-manager.h"
 #include "salut-discovery-client.h"
 #include "tube-stream.h"
+#include "tube-dbus.h"
 
 #include <telepathy-glib/channel-manager.h>
 #include <telepathy-glib/dbus.h>
@@ -322,12 +323,27 @@ salut_muc_manager_foreach_channel (TpChannelManager *iface,
   SalutMucManager *fac = SALUT_MUC_MANAGER (iface);
   SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (fac);
   struct _ForeachData data;
+  GHashTableIter iter;
+  gpointer value;
 
   data.user_data = user_data;
   data.foreach = foreach;
 
   g_hash_table_foreach (priv->text_channels, _foreach_slave, &data);
-  g_hash_table_foreach (priv->tubes_channels, _foreach_slave, &data);
+
+  g_hash_table_iter_init (&iter, priv->tubes_channels);
+  while (g_hash_table_iter_next (&iter, NULL, &value))
+    {
+      TpExportableChannel *chan = TP_EXPORTABLE_CHANNEL (value);
+
+      /* Add channels of type Channel.Type.Tubes */
+      foreach (chan, user_data);
+
+      /* Add channels of type Channel.Type.{Stream|DBus}Tube which live in the
+       * SalutTubesChannel object */
+      salut_tubes_channel_foreach (SALUT_TUBES_CHANNEL (chan), foreach,
+          user_data);
+    }
 }
 
 static const gchar * const muc_channel_fixed_properties[] = {
@@ -380,10 +396,16 @@ salut_muc_manager_foreach_channel_class (TpChannelManager *manager,
 
   /* org.freedesktop.Telepathy.Channel.Type.StreamTube */
   g_value_set_static_string (channel_type_value,
-      SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE);
+      TP_IFACE_CHANNEL_TYPE_STREAM_TUBE);
   func (manager, table, salut_tube_stream_channel_get_allowed_properties (),
       user_data);
 
+  /* Muc Channel.Type.DBusTube */
+  g_value_set_static_string (channel_type_value,
+      TP_IFACE_CHANNEL_TYPE_DBUS_TUBE);
+  func (manager, table, salut_tube_dbus_channel_get_allowed_properties (),
+      user_data);
+
   g_hash_table_destroy (table);
 }
 
@@ -707,6 +729,128 @@ create_tubes_channel (SalutMucManager *self,
   return tubes_chan;
 }
 
+static gboolean
+handle_tube_channel_request (SalutMucManager *self,
+                             gpointer request_token,
+                             GHashTable *request_properties,
+                             gboolean require_new,
+                             TpHandle handle,
+                             GError **error)
+{
+  SalutMucManagerPrivate *priv = SALUT_MUC_MANAGER_GET_PRIVATE (self);
+  TpBaseConnection *base_conn = (TpBaseConnection *) priv->connection;
+  SalutMucChannel *text_chan;
+  SalutTubesChannel *tubes_chan;
+  SalutTubeIface *new_channel;
+  GHashTable *channels;
+  GSList *request_tokens;
+  gboolean announce_text = FALSE, announce_tubes = FALSE;
+
+  tubes_chan = g_hash_table_lookup (priv->tubes_channels,
+      GUINT_TO_POINTER (handle));
+  if (tubes_chan == NULL)
+    {
+      tubes_chan = create_tubes_channel (self, handle,
+          base_conn->self_handle, NULL, FALSE, &announce_text,
+          FALSE, error);
+      if (tubes_chan == NULL)
+        return FALSE;
+      announce_tubes = TRUE;
+    }
+
+  g_assert (tubes_chan != NULL);
+  new_channel = salut_tubes_channel_tube_request (tubes_chan, request_token,
+      request_properties, require_new);
+  g_assert (new_channel != NULL);
+
+  /* announce channels */
+  channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+      NULL, NULL);
+
+  if (announce_text)
+    {
+      text_chan = g_hash_table_lookup (priv->text_channels,
+          GINT_TO_POINTER (handle));
+      g_assert (text_chan != NULL);
+      g_hash_table_insert (channels, text_chan, NULL);
+    }
+
+  if (announce_tubes)
+    {
+      g_hash_table_insert (channels, tubes_chan, NULL);
+    }
+
+  request_tokens = g_slist_prepend (NULL, request_token);
+  g_hash_table_insert (channels, new_channel, request_tokens);
+  tp_channel_manager_emit_new_channels (self, channels);
+
+  g_hash_table_destroy (channels);
+  g_slist_free (request_tokens);
+  return TRUE;
+}
+
+static gboolean
+handle_stream_tube_channel_request (SalutMucManager *self,
+    gpointer request_token,
+    GHashTable *request_properties,
+    gboolean require_new,
+    TpHandle handle,
+    GError **error)
+{
+  const gchar *service;
+
+  if (tp_channel_manager_asv_has_unknown_properties (request_properties,
+          muc_tubes_channel_fixed_properties,
+          salut_tube_stream_channel_get_allowed_properties (),
+          error))
+    return FALSE;
+
+  /* "Service" is a mandatory, not-fixed property */
+  service = tp_asv_get_string (request_properties,
+            TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+  if (service == NULL)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+          "Request does not contain the mandatory property '%s'",
+          TP_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
+      return FALSE;
+    }
+
+  return handle_tube_channel_request (self, request_token, request_properties,
+      require_new, handle, error);
+}
+
+static gboolean
+handle_dbus_tube_channel_request (SalutMucManager *self,
+    gpointer request_token,
+    GHashTable *request_properties,
+    gboolean require_new,
+    TpHandle handle,
+    GError **error)
+{
+  const gchar *service;
+
+  if (tp_channel_manager_asv_has_unknown_properties (request_properties,
+          muc_tubes_channel_fixed_properties,
+          salut_tube_dbus_channel_get_allowed_properties (),
+          error))
+    return FALSE;
+
+  /* "ServiceName" is a mandatory, not-fixed property */
+  service = tp_asv_get_string (request_properties,
+      TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+  if (service == NULL)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
+          "Request does not contain the mandatory property '%s'",
+          TP_IFACE_CHANNEL_TYPE_DBUS_TUBE ".ServiceName");
+      return FALSE;
+    }
+
+  return handle_tube_channel_request (self, request_token, request_properties,
+      require_new, handle, error);
+}
+
 static gboolean
 salut_muc_manager_request (SalutMucManager *self,
                            gpointer request_token,
@@ -730,7 +874,8 @@ salut_muc_manager_request (SalutMucManager *self,
 
   if (tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TEXT) &&
       tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_TUBES) &&
-      tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE) &&
+      tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
     return FALSE;
 
   /* validity already checked by TpBaseConnection */
@@ -805,72 +950,17 @@ salut_muc_manager_request (SalutMucManager *self,
 
       return TRUE;
     }
-  else if (!tp_strdiff (channel_type, SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE))
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
     {
-      const gchar *service;
-      SalutTubeIface *new_channel;
-      GHashTable *channels;
-      GSList *request_tokens;
-      gboolean announce_text = FALSE, announce_tubes = FALSE;
-
-      if (tp_channel_manager_asv_has_unknown_properties (request_properties,
-              muc_tubes_channel_fixed_properties,
-              salut_tube_stream_channel_get_allowed_properties (),
-              &error))
-        goto error;
-
-      /* "Service" is a mandatory, not-fixed property */
-      service = tp_asv_get_string (request_properties,
-                SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
-      if (service == NULL)
-        {
-          g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_IMPLEMENTED,
-              "Request does not contain the mandatory property '%s'",
-              SALUT_IFACE_CHANNEL_TYPE_STREAM_TUBE ".Service");
-          goto error;
-        }
-
-      tubes_chan = g_hash_table_lookup (priv->tubes_channels,
-          GUINT_TO_POINTER (handle));
-      if (tubes_chan == NULL)
-        {
-          tubes_chan = create_tubes_channel (self, handle,
-              base_conn->self_handle, NULL, FALSE, &announce_text,
-              FALSE, &error);
-          if (tubes_chan == NULL)
-            goto error;
-          announce_tubes = TRUE;
-        }
-
-      g_assert (tubes_chan != NULL);
-      new_channel = salut_tubes_channel_tube_request (tubes_chan, channel_type,
-          service);
-      g_assert (new_channel != NULL);
-
-      /* announce channels */
-      channels = g_hash_table_new_full (g_direct_hash, g_direct_equal,
-          NULL, NULL);
-
-      if (announce_text)
-        {
-          text_chan = g_hash_table_lookup (priv->text_channels,
-              GINT_TO_POINTER (handle));
-          g_assert (text_chan != NULL);
-          g_hash_table_insert (channels, text_chan, NULL);
-        }
-
-      if (announce_tubes)
-        {
-          g_hash_table_insert (channels, tubes_chan, NULL);
-        }
-
-      request_tokens = g_slist_prepend (NULL, request_token);
-      g_hash_table_insert (channels, new_channel, request_tokens);
-      tp_channel_manager_emit_new_channels (self, channels);
-
-      g_hash_table_destroy (channels);
-      g_slist_free (request_tokens);
-      return TRUE;
+      if (handle_stream_tube_channel_request (self, request_token,
+          request_properties, require_new, handle, &error))
+        return TRUE;
+    }
+  else if (!tp_strdiff (channel_type, TP_IFACE_CHANNEL_TYPE_DBUS_TUBE))
+    {
+      if (handle_dbus_tube_channel_request (self, request_token,
+          request_properties, require_new, handle, &error))
+        return TRUE;
     }
   else
     {
